from base_st_classes import *

class GetattrSuite(NFSSuite):
    """Test operation 9: GETATTR

    FIXME: Add attribute directory and named attribute testing. 

    Equivalence partitioning:

    Input Condition: current filehandle
        Valid equivalence classes:
            file(1)
            link(2)
            block(3)
            char(4)
            socket(5)
            FIFO(6)
            dir(7)
        Invalid equivalence classes:
            invalid filehandle(8)
    Input Condition: attrbits
        Valid equivalence classes:
            all requests without FATTR4_*_SET (9)
        Invalid equivalence classes:
            requests with FATTR4_*_SET (10)
    
    """
    # NOTE - getattr returns OK for almost anything.  Really need to
    # check return values for these tests.
    
    #
    # Testcases covering valid equivalence classes.
    #
    def testAllObjects(self):
        """GETATTR(FATTR4_SIZE) on all type of objects

        Covered valid equivalence classes: 1, 2, 3, 4, 5, 6, 7, 9
        """
        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr([FATTR4_SIZE]))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)
            obj = res.resarray[-1].arm.arm.obj_attributes
            d = nfs4lib.fattr2dict(obj)
            if len(d)!=1 or (FATTR4_SIZE not in d):
                self.fail("Bad return %s"%d)

    def xxxtestMountedOnFileid(self):
        """GETATTR(FATTR4_MOUNTED_ON_FILEID)

        This DOES NOT work on standard test tree.  It assumes that / and /user
        exist, and that /user is a mountpoint.
        """
        
        lookupops = [self.ncl.lookup_op("user")]
        ops = [self.ncl.putrootfh_op()] + lookupops
        ops.append(self.ncl.getattr([FATTR4_MOUNTED_ON_FILEID,FATTR4_FILEID]))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        obj = res.resarray[-1].arm.arm.obj_attributes
        d = nfs4lib.fattr2dict(obj)
        print
        print "From Getattr /user - ",d

        ops = [self.ncl.putrootfh_op()]
        attrmask = nfs4lib.list2attrmask([FATTR4_MOUNTED_ON_FILEID,FATTR4_FILEID])
        ops.append(self.ncl.readdir(attr_request=attrmask))
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        # Find 'user' (note this assumes dir listing is small enough that
        # the whole listing was returned)
        reply = res.resarray[-1].arm.arm.reply
        entry = reply.entries[0]
        while 1:
            if entry.name=="user":
                break
            if not entry.nextentry:
                self.fail("Could not find mountpoint /user")
            entry = entry.nextentry[0]
        d2 = nfs4lib.fattr2dict(entry.attrs)
        print "From Readdir / - ",d2
        

    #
    # Testcases covering invalid equivalence classes.
    #
    def testNoFh(self):
        """GETATTR should fail with NFS4ERR_NOFILEHANDLE if no (cfh)

        Covered invalid equivalence classes: 8
        """
        
        getattrop = self.ncl.getattr([FATTR4_SIZE])
        res = self.ncl.do_ops([getattrop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

    def testWriteOnlyAttributes(self):
        """GETATTR(FATTR4_*_SET) should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 10

        Comments: Some attributes are write-only (currently
        FATTR4_TIME_ACCESS_SET and FATTR4_TIME_MODIFY_SET). If GETATTR
        is called with any of these, NFS4ERR_INVAL should be returned.
        """
        for attr in [FATTR4_TIME_ACCESS_SET, FATTR4_TIME_MODIFY_SET] :
            lookupops = self.ncl.lookup_path(self.regfile)
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr([attr]))
            res = self.ncl.do_ops(ops)
            self.assert_status(res, [NFS4ERR_INVAL])

    #
    # Extra tests. 
    #
    def testAllMandatory(self):
        """GETATTR should return all mandatory attributes

        Extra test

        Comments: A server should be able to return all mandatory
        attributes.
        """

        attrbitnum_dict = nfs4lib.get_attrbitnum_dict()
        all_mandatory_names = [
            "supported_attrs", 
            "type",
            "fh_expire_type",
            "change",
            "size",
            "link_support",
            "symlink_support",
            "named_attr",
            "fsid",
            "unique_handles",
            "lease_time",
            "rdattr_error",
            "filehandle"]
        all_mandatory = []
        
        for attrname in all_mandatory_names:
            all_mandatory.append(attrbitnum_dict[attrname])

        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr(all_mandatory))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)

            obj = res.resarray[-1].arm.arm.obj_attributes
            d = nfs4lib.fattr2dict(obj)
            unsupported = []
            keys = d.keys()
            for attr in all_mandatory:
                if not attr in keys:
                    attrname = nfs4lib.get_bitnumattr_dict()[attr]
                    unsupported.append(attrname)

            if unsupported:
                self.fail("mandatory attributes not supported: %s" % str(unsupported))

    def testUnknownAttr(self):
        """GETATTR should not fail on unknown attributes

        Covered valid equivalence classes: 1, 9

        Comments: This test calls GETATTR with request for attribute
        number 1000.  Servers should not fail on unknown attributes.
        """
        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr([1000]))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)
            obj = res.resarray[-1].arm.arm.obj_attributes
            d = nfs4lib.fattr2dict(obj)
            if d:
                self.fail("Returned a value %s" % d)

    def testEmptyCall(self):
        """GETATTR should accept empty request

        Covered valid equivalence classes: 1, 9

        Comments: GETATTR should accept empty request
        """
        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr([]))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)
            obj = res.resarray[-1].arm.arm.obj_attributes
            d = nfs4lib.fattr2dict(obj)
            if d:
                self.fail("Returned a value %s" % d)

    def testSupported(self):
        """GETATTR(FATTR4_SUPPORTED_ATTRS) should return all mandatory

        Covered valid equivalence classes: 1, 9
        
        Comments: GETATTR(FATTR4_SUPPORTED_ATTRS) should return at
        least all mandatory attributes
        """
        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr([FATTR4_SUPPORTED_ATTRS]))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)

            obj = res.resarray[-1].arm.arm.obj_attributes
            d = nfs4lib.fattr2dict(obj)
            i = nfs4lib.intlist2long(d[FATTR4_SUPPORTED_ATTRS])

            all_mandatory_bits = (2**(FATTR4_RDATTR_ERROR+1) - 1) + (2**(FATTR4_FILEHANDLE))
            returned_mandatories = i & all_mandatory_bits
            self.failIf(not returned_mandatories == all_mandatory_bits,
                        "not all mandatory attributes returned: %s" % \
                        nfs4lib.int2binstring(returned_mandatories)[-12:])

    def testLongList(self):
        """GETATTR should not fail when ask for all attributes

        """
        all = range(56)
        all.remove(FATTR4_TIME_ACCESS_SET)
        all.remove(FATTR4_TIME_MODIFY_SET)
        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr(all))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)
            obj = res.resarray[-1].arm.arm.obj_attributes
            # We do this to make sure everything came through uncorrupted
            d = nfs4lib.fattr2dict(obj)

    def testTimeCreate(self):
        """GETATTR should not fail when ask for FATTR4_TIME_CREATE

        """
        for lookupops in self.ncl.lookup_all_objects():
            ops = [self.ncl.putrootfh_op()] + lookupops
            ops.append(self.ncl.getattr([FATTR4_TIME_CREATE]))
            res = self.ncl.do_ops(ops)
            self.assert_OK(res)
            obj = res.resarray[-1].arm.arm.obj_attributes
            # We do this to make sure everything came through uncorrupted
            d = nfs4lib.fattr2dict(obj)

